__version__ = "0.1.13"
__author__ = "Mehdi Samsami"

__all__ = [
    'GeneralNB',
    'GaussianWNB',
    'Distribution',
    'ContinuousDistMixin',
    'DiscreteDistMixin'
]

from ._base import ContinuousDistMixin, DiscreteDistMixin
from ._enums import Distribution
from .gnb import GeneralNB
from .gwnb import GaussianWNB
